@@ -1,6 +1,5 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import hashlib
 from decimal import Decimal
 
 from sql import Null
@@ -12,12 +11,13 @@ from trytond import backend
 from trytond.i18n import gettext
 from trytond.pool import Pool, PoolMeta
 from trytond.model import ModelSQL, ModelView, fields
-from trytond.pyson import Eval, If, Bool
+from trytond.pyson import Eval, Id
 from trytond.wizard import (Wizard, StateView, StateAction, StateTransition,
     Button)
 from trytond.transaction import Transaction
 from trytond.tools.multivalue import migrate_property
 from trytond.modules.company.model import CompanyValueMixin
+from trytond.modules.currency.fields import Monetary
 
 from .exceptions import BlockedWarning, GroupWarning
 from .payment import KINDS
@@ -25,17 +25,14 @@ from .payment import KINDS
 
 class MoveLine(metaclass=PoolMeta):
     __name__ = 'account.move.line'
-    payment_amount = fields.Function(fields.Numeric('Payment Amount',
-            digits=(16,
-                If(Bool(Eval('second_currency_digits')),
-                    Eval('second_currency_digits', 2),
-                    Eval('currency_digits', 2))),
+    payment_amount = fields.Function(Monetary(
+            "Payment Amount",
+            currency='payment_currency', digits='payment_currency',
             states={
                 'invisible': ~Eval('payment_kind'),
                 },
-            depends=['payment_kind', 'second_currency_digits',
-                'currency_digits']), 'get_payment_amount',
-        searcher='search_payment_amount')
+            depends=['payment_kind']),
+        'get_payment_amount', searcher='search_payment_amount')
     payment_currency = fields.Function(fields.Many2One(
             'currency.currency', "Payment Currency"),
         'get_payment_currency')
@@ -121,7 +118,8 @@ class MoveLine(metaclass=PoolMeta):
         second_amount = Abs(table.amount_second_currency) - payment_amount
         amount = Case((table.second_currency == Null, main_amount),
             else_=second_amount)
-        value = cls.payment_amount.sql_format(value)
+        value = cls.payment_amount._field.sql_cast(
+            cls.payment_amount.sql_format(value))
 
         query = (table
             .join(payment, type_='LEFT',
@@ -188,6 +186,80 @@ class MoveLine(metaclass=PoolMeta):
                 'payment_blocked': False,
                 })
 
+    @classmethod
+    def _pay_direct_debit_domain(cls, date):
+        return [
+            ['OR',
+                ('account.type.receivable', '=', True),
+                ('account.type.payable', '=', True),
+                ],
+            ('party', '!=', None),
+            ('reconciliation', '=', None),
+            ('payment_amount', '!=', 0),
+            ('move_state', '=', 'posted'),
+            ['OR',
+                ('debit', '>', 0),
+                ('credit', '<', 0),
+                ],
+            ('maturity_date', '<=', date),
+            ]
+
+    @classmethod
+    def pay_direct_debit(cls, date=None):
+        pool = Pool()
+        Date = pool.get('ir.date')
+        Payment = pool.get('account.payment')
+        Reception = pool.get('party.party.reception_direct_debit')
+        if date is None:
+            date = Date.today()
+        with Transaction().set_context(_check_access=True):
+            lines = cls.search(cls._pay_direct_debit_domain(date))
+
+        payments = []
+        for line in lines:
+            if not line.payment_amount:
+                # SQLite fails to search for payment_amount != 0
+                continue
+            pattern = Reception.get_pattern(line)
+            for reception in line.party.reception_direct_debits:
+                if reception.match(pattern):
+                    payments.extend(reception.get_payments(line))
+        Payment.save(payments)
+        return payments
+
+
+class CreateDirectDebit(Wizard):
+    "Create Direct Debit"
+    __name__ = 'account.move.line.create_direct_debit'
+
+    start = StateView('account.move.line.create_direct_debit.start',
+        'account_payment.move_line_create_direct_debit_view_form', [
+            Button("Cancel", 'end', 'tryton-cancel'),
+            Button("Create", 'create_', 'tryton-ok', default=True),
+            ])
+    create_ = StateAction('account_payment.act_payment_form')
+
+    def do_create_(self, action):
+        pool = Pool()
+        Line = pool.get('account.move.line')
+        payments = Line.pay_direct_debit(date=self.start.date)
+        return action, {
+            'res_id': [p.id for p in payments],
+            }
+
+
+class CreateDirectDebitStart(ModelView):
+    "Create Direct Debit"
+    __name__ = 'account.move.line.create_direct_debit.start'
+
+    date = fields.Date(
+        "Date", required=True,
+        help="Create direct debit for lines due up to this date.")
+
+    @classmethod
+    def default_date(cls):
+        return Pool().get('ir.date').today()
+
 
 class PayLineStart(ModelView):
     "Pay Line"
@@ -237,46 +309,48 @@ class PayLine(Wizard):
         Warning = pool.get('res.user.warning')
 
         reverse = {'receivable': 'payable', 'payable': 'receivable'}
-        types = {
-            kind: {
-                'parties': set(),
-                'lines': list(),
-                }
-            for kind in reverse.keys()}
+        companies = {}
         lines = self.records
         for line in lines:
+            types = companies.setdefault(line.move.company, {
+                    kind: {
+                        'parties': set(),
+                        'lines': list(),
+                        }
+                    for kind in reverse.keys()})
             for kind in types:
                 if getattr(line.account.type, kind):
                     types[kind]['parties'].add(line.party)
                     types[kind]['lines'].append(line)
 
-        for kind in types:
-            parties = types[kind]['parties']
-            others = Line.search([
-                    ('account.type.' + reverse[kind], '=', True),
-                    ('party', 'in', [p.id for p in parties]),
-                    ('reconciliation', '=', None),
-                    ('payment_amount', '!=', 0),
-                    ('move_state', '=', 'posted'),
-                    ])
-            for party in parties:
-                party_lines = [l for l in others if l.party == party]
-                if not party_lines:
-                    continue
-                lines = [l for l in types[kind]['lines']
-                    if l.party == party]
-                warning_name = '%s:%s:%s' % (
-                    reverse[kind], party,
-                    hashlib.md5(str(lines).encode('utf-8')).hexdigest())
-                if Warning.check(warning_name):
-                    names = ', '.join(l.rec_name for l in lines[:5])
-                    if len(lines) > 5:
-                        names += '...'
-                    raise GroupWarning(warning_name,
-                        gettext('account_payment.msg_pay_line_group',
-                            names=names,
-                            party=party.rec_name,
-                            line=party_lines[0].rec_name))
+        for company, types in companies.items():
+            for kind in types:
+                parties = types[kind]['parties']
+                others = Line.search([
+                        ('move.company', '=', company.id),
+                        ('account.type.' + reverse[kind], '=', True),
+                        ('party', 'in', [p.id for p in parties]),
+                        ('reconciliation', '=', None),
+                        ('payment_amount', '!=', 0),
+                        ('move_state', '=', 'posted'),
+                        ])
+                for party in parties:
+                    party_lines = [l for l in others if l.party == party]
+                    if not party_lines:
+                        continue
+                    lines = [l for l in types[kind]['lines']
+                        if l.party == party]
+                    warning_name = Warning.format(
+                        '%s:%s' % (reverse[kind], party), lines)
+                    if Warning.check(warning_name):
+                        names = ', '.join(l.rec_name for l in lines[:5])
+                        if len(lines) > 5:
+                            names += '...'
+                        raise GroupWarning(warning_name,
+                            gettext('account_payment.msg_pay_line_group',
+                                names=names,
+                                party=party.rec_name,
+                                line=party_lines[0].rec_name))
         return {}
 
     def _get_journals(self):
@@ -384,7 +458,9 @@ class Configuration(metaclass=PoolMeta):
             domain=[
                 ('company', 'in',
                     [Eval('context', {}).get('company', -1), None]),
-                ('code', '=', 'account.payment.group'),
+                ('sequence_type', '=',
+                    Id('account_payment',
+                        'sequence_type_account_payment_group')),
                 ]))
 
     @classmethod
@@ -400,7 +476,8 @@ class ConfigurationPaymentGroupSequence(ModelSQL, CompanyValueMixin):
         'ir.sequence', "Payment Group Sequence", required=True,
         domain=[
             ('company', 'in', [Eval('company', -1), None]),
-            ('code', '=', 'account.payment.group'),
+            ('sequence_type', '=',
+                Id('account_payment', 'sequence_type_account_payment_group')),
             ],
         depends=['company'])
 
